/*
 * Copyright (c) 2019 Samsung Electronics Co., Ltd.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#include "screen_connector_common/export.h"
#include "screen_connector_launcher_service_evas/include/screen_connector_launcher_service_evas.h"
#include "screen_connector_launcher_service_evas/launcher_service_evas.h"
#include "screen_connector_launcher_service_evas/log_internal.h"

namespace {

static screen_connector_launcher_service_evas_error_e __convert_error(
    screen_connector::Error error) {
  if (error == screen_connector::Error::Disqualified) {
    return SCREEN_CONNECTOR_LAUNCHER_SERVICE_EVAS_ERROR_DISQUALIFIED;
  } else if (error == screen_connector::Error::WrongRequest) {
    return SCREEN_CONNECTOR_LAUNCHER_SERVICE_EVAS_ERROR_WRONG_REQUEST;
  } else {
    return SCREEN_CONNECTOR_LAUNCHER_SERVICE_EVAS_ERROR_NONE;
  }
}

static screen_connector_launcher_service_evas_direction_e __convert_direction(
    screen_connector::Direction direction) {
  if (direction == screen_connector::Direction::Forward)
    return SCREEN_CONNECTOR_LAUNCHER_SERVICE_EVAS_DIRECTION_FORWARD;

  return SCREEN_CONNECTOR_LAUNCHER_SERVICE_EVAS_DIRECTION_BACKWARD;
}

static screen_connector_launcher_service_evas_image_type_e __convert_image_type(
    screen_connector::ImageType image_type) {
  if (image_type == screen_connector::ImageType::RemoteSurfaceTBM) {
    return SCREEN_CONNECTOR_LAUNCHER_SERVICE_EVAS_IMAGE_TYPE_REMOTE_SURFACE_TBM;
  } else if (image_type == screen_connector::ImageType::RemoteSurfaceImageFile) {
    return SCREEN_CONNECTOR_LAUNCHER_SERVICE_EVAS_IMAGE_TYPE_REMOTE_SURFACE_IMAGE_FILE;
  } else if (image_type == screen_connector::ImageType::SplashScreenImage) {
    return SCREEN_CONNECTOR_LAUNCHER_SERVICE_EVAS_IMAGE_TYPE_SPLASH_SCREEN_IMAGE;
  } else {
    return SCREEN_CONNECTOR_LAUNCHER_SERVICE_EVAS_IMAGE_TYPE_SPLASH_SCREEN_EDJE;
  }
}

class LauncherServiceEvasStub : public screen_connector::LauncherServiceEvas {
 public:
  LauncherServiceEvasStub(std::shared_ptr<screen_connector::EvasObject> win)
    : screen_connector::LauncherServiceEvas(std::move(win)) {
  }

  void SetOps(screen_connector_launcher_service_evas_ops* ops,
              void* user_data) {
    ops_ = *ops;
    user_data_ = user_data;
  }

  void OnEvasPrepareEvent(const screen_connector::EvasObject& image,
                          screen_connector::ImageType image_type,
                          screen_connector::Direction direction,
                          int32_t x,
                          int32_t y,
                          const std::string& shared_widget_info,
                          uint32_t serial) override {
    if (ops_.prepare) {
      ops_.prepare(image.GetRaw(),
          __convert_image_type(image_type),
          __convert_direction(direction),
          x,
          y,
          shared_widget_info.c_str(),
          serial,
          user_data_);
    }
  }

  void OnEvasStopEvent(uint32_t serial) override {
    if (ops_.stop) {
      ops_.stop(serial, user_data_);
    }
  }

  void OnEvasErrorEvent(screen_connector::Error error,
                        uint32_t serial) override {
    if (ops_.error) {
      ops_.error(__convert_error(error), serial, user_data_);
    }
  }

  void OnEvasCleanUpEvent(uint32_t serial) override {
    if (ops_.cleanup) {
      ops_.cleanup(serial, user_data_);
    }
  }

 private:
  screen_connector_launcher_service_evas_ops ops_ = { 0, };
  void* user_data_ = nullptr;
};

}  // namespace

extern "C" EXPORT int screen_connector_launcher_service_evas_create(
    Evas_Object* win,
    screen_connector_launcher_service_evas_h* handle) {
  if (win == nullptr || handle == nullptr) {
    _E("Invalid parameter");
    return -EINVAL;
  }

  auto ls = new (std::nothrow) LauncherServiceEvasStub(
      std::shared_ptr<screen_connector::EvasObject>(
          new (std::nothrow) screen_connector::EvasObject(win, false)));

  *handle = static_cast<void*>(ls);

  return 0;
}

extern "C" EXPORT int screen_connector_launcher_service_evas_destroy(
    screen_connector_launcher_service_evas_h handle) {
  if (handle == nullptr) {
    _E("Invalid parameter");
    return -EINVAL;
  }

  auto ls = static_cast<LauncherServiceEvasStub*>(handle);
  delete ls;

  return 0;
}

extern "C" EXPORT int screen_connector_launcher_service_evas_set_ops(
    screen_connector_launcher_service_evas_h handle,
    screen_connector_launcher_service_evas_ops* ops,
    void* user_data) {
  if (handle == nullptr || ops == nullptr) {
    _E("Invalid parameter");
    return -EINVAL;
  }

  auto ls = static_cast<LauncherServiceEvasStub*>(handle);
  ls->SetOps(ops, user_data);

  return 0;
}

extern "C" EXPORT int screen_connector_launcher_service_evas_launch(
    screen_connector_launcher_service_evas_h handle,
    const char* app_id,
    const char* inst_id,
    int pid,
    uint32_t serial) {
 if (handle == nullptr || app_id == nullptr || pid < -1) {
    _E("Invalid parameter");
    return -EINVAL;
  }

  auto ls = static_cast<LauncherServiceEvasStub*>(handle);
  return ls->Launch(app_id, inst_id, pid, serial);
}

extern "C" EXPORT int
screen_connector_launcher_service_evas_launch_with_shared_widget (
    screen_connector_launcher_service_evas_h handle,
    const char* app_id,
    const char* inst_id,
    int pid,
    uint32_t serial) {
 if (handle == nullptr || app_id == nullptr || pid < -1) {
    _E("Invalid parameter");
    return -EINVAL;
  }

  auto ls = static_cast<LauncherServiceEvasStub*>(handle);
  return ls->LaunchWithSharedWidget(app_id, inst_id, pid, serial);
}

extern "C" EXPORT int screen_connector_launcher_service_evas_launching(
    screen_connector_launcher_service_evas_h handle,
    uint32_t serial) {
  if (handle == nullptr) {
    _E("Invalid parameter");
    return -EINVAL;
  }

  auto ls = static_cast<LauncherServiceEvasStub*>(handle);
  return ls->Launching(serial);
}

extern "C" EXPORT int screen_connector_launcher_service_evas_launch_cancel(
    screen_connector_launcher_service_evas_h handle,
    uint32_t serial) {
  if (handle == nullptr) {
    _E("Invalid parameter");
    return -EINVAL;
  }

  auto ls = static_cast<LauncherServiceEvasStub*>(handle);
  return ls->LaunchCancel(serial);
}

extern "C" EXPORT int screen_connector_launcher_service_evas_launch_done(
    screen_connector_launcher_service_evas_h handle,
    uint32_t serial) {
  if (handle == nullptr) {
    _E("Invalid parameter");
    return -EINVAL;
  }

  auto ls = static_cast<LauncherServiceEvasStub*>(handle);
  return ls->LaunchDone(serial);
}
