/*
 * Copyright (c) 2019 Samsung Electronics Co., Ltd.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#include "screen_connector_common/export.h"
#include "screen_connector_common/wl2_window.h"
#include "screen_connector_shared_widget_launch/include/screen_connector_shared_widget_launch.h"
#include "screen_connector_shared_widget_launch/log_internal.h"
#include "screen_connector_shared_widget_launch/shared_widget_launch.h"

namespace {

class SharedWidgetLaunchStub : public screen_connector::SharedWidgetLaunch,
  public screen_connector::SharedWidgetLaunch::IEventListener {
 public:
  explicit SharedWidgetLaunchStub(
      std::shared_ptr<screen_connector::Wl2Window> wl2_win)
    : screen_connector::SharedWidgetLaunch(std::move(wl2_win), this) {
  }

  virtual ~SharedWidgetLaunchStub() = default;

  void SetPrepareCB(screen_connector_shared_widget_launch_prepare_cb cb,
      void* user_data) {
    prepare_cb_ = cb;
    user_data_ = user_data;
  }

  void OnPrepareEvent(screen_connector::PrepareState state,
                      uint32_t serial) override {
    if (!prepare_cb_)
      return;

    screen_connector_shared_widget_launch_prepare_state_e prepare_state =
      static_cast<screen_connector_shared_widget_launch_prepare_state_e>(state);

    prepare_cb_(prepare_state, serial, user_data_);
  }

 private:
  screen_connector_shared_widget_launch_prepare_cb prepare_cb_ = nullptr;
  void* user_data_ = nullptr;
};

extern "C" EXPORT int screen_connector_shared_widget_launch_create(
    Ecore_Wl2_Window* wl2_win,
    screen_connector_shared_widget_launch_h* handle) {
  if (wl2_win == nullptr || handle == nullptr) {
    _E("Invalid parameter");
    return -EINVAL;
  }

  auto swl = new (std::nothrow) SharedWidgetLaunchStub(
      std::shared_ptr<screen_connector::Wl2Window>(
          new (std::nothrow) screen_connector::Wl2Window(wl2_win)));

  *handle = static_cast<void*>(swl);

  return 0;
}

extern "C" EXPORT int screen_connector_shared_widget_launch_destroy(
    screen_connector_shared_widget_launch_h handle) {
  if (handle == nullptr) {
    _E("Invalid parameter");
    return -EINVAL;
  }

  auto swl = static_cast<SharedWidgetLaunchStub*>(handle);
  delete swl;

  return 0;
}

extern "C" EXPORT int screen_connector_shared_widget_launch_prepare_cb_set(
    screen_connector_shared_widget_launch_h handle,
    screen_connector_shared_widget_launch_prepare_cb callback,
    void* user_data) {
  if (handle == nullptr || callback == nullptr) {
    _E("Invalid parameter");
    return -EINVAL;
  }

  auto swl = static_cast<SharedWidgetLaunchStub*>(handle);
  swl->SetPrepareCB(callback, user_data);

  return 0;
}

extern "C" EXPORT int screen_connector_shared_widget_launch_prepare_done(
    screen_connector_shared_widget_launch_h handle,
    const char* shared_widget_info,
    screen_connector_shared_widget_launch_prepare_state_e state,
    uint32_t serial) {
  if (handle == nullptr || shared_widget_info == nullptr) {
    _E("Invalid parameter");
    return -EINVAL;
  }

  auto swl = static_cast<SharedWidgetLaunchStub*>(handle);
  swl->PrepareDone(shared_widget_info,
      static_cast<screen_connector::PrepareState>(state), serial);

  return 0;
}

}  // namespace
