/*
 * Copyright (c) 2016 Samsung Electronics Co., Ltd.
 *
 * Licensed under the Apache License, Version 2.0 (the License);
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#include <stdio.h>
#include <errno.h>
#include <glib.h>
#include <sys/mman.h>
#include <dlog.h>
#include <Elementary.h>
#include <Ecore_Wl2.h>
#include <wayland-extension/tizen-extension-client-protocol.h>
#include <tizen-remote-surface-client-protocol.h>
#include <tbm_surface.h>
#include <tbm_surface_internal.h>
#include <uuid/uuid.h>

#include <map>
#include <list>
#include <algorithm>
#include <memory>

#include "screen_connector_watcher_evas.h"
#include "screen_connector_remote_surface_evas/remote_surface_watcher.h"

#ifdef LOG_TAG
#undef LOG_TAG
#endif

#define LOG_TAG "SC_WATCHER_EVAS"

class RemoteSurfaceWatcherStub : public screen_connector::RemoteSurfaceWatcher {
 public:
  RemoteSurfaceWatcherStub(screen_connector::RemoteSurface::Type type,
                           screen_connector_watcher_evas_ops* ops,
                           void* data,
                           std::shared_ptr<screen_connector::EvasObject> win)
      : screen_connector::RemoteSurfaceWatcher(type, win),
        ops_(*ops), data_(data) {
  }

  void OnWatcherAdded(const std::string& appId,
                      const std::string& instId, int pid) override {
    if (ops_.added_cb)
      ops_.added_cb(appId.c_str(), instId.c_str(), pid, data_);
  }

  void OnWatcherRemoved(const std::string& appId,
                        const std::string& instId, int pid) override {
    if (ops_.removed_cb)
      ops_.removed_cb(appId.c_str(), instId.c_str(), pid, data_);
  }

  void OnWatcherChanged(const std::string& appId, const std::string& instId,
                        int pid,
                        const screen_connector::EvasObject& image) override {
    if (ops_.updated_cb) {
      ops_.updated_cb(appId.c_str(), instId.c_str(), pid,
          image.GetRaw(), data_);
    }
  }

 private:
  screen_connector_watcher_evas_ops ops_;
  void* data_;
};

class WatcherConnector {
 public:
  explicit WatcherConnector(Evas_Object* win) : win_(win) { }
  ~WatcherConnector() { }

  void Add(RemoteSurfaceWatcherStub* rs) {
    surfaces_.emplace_back(rs);
  }

  void Remove(RemoteSurfaceWatcherStub* rs) {
    for (auto& i : surfaces_) {
      if (i.get() == rs) {
        surfaces_.remove(i);
        break;
      }
    }
  }

  Evas_Object* GetWin() const { return win_; }
  int GetCount() const { return surfaces_.size(); }

 private:
  Evas_Object *win_;
  std::list<std::unique_ptr<RemoteSurfaceWatcherStub>> surfaces_;
};

static WatcherConnector* __connector;

extern "C" EXPORT_API int
screen_connector_watcher_evas_init(Evas_Object* win) {
  screen_connector::RemoteSurface::InitManager();
  __connector = new WatcherConnector(win);

  return 0;
}

extern "C" EXPORT_API int
screen_connector_watcher_evas_fini(void) {
  delete __connector;
  screen_connector::RemoteSurface::FiniManager();

  return 0;
}

extern "C" EXPORT_API screen_connector_watcher_evas_h
screen_connector_watcher_evas_add(screen_connector_watcher_evas_ops* ops,
                                  void* data) {
  auto rs = new RemoteSurfaceWatcherStub(
      screen_connector::RemoteSurface::Type::UI, ops, data,
      std::shared_ptr<screen_connector::EvasObject>(
      new screen_connector::EvasObject(__connector->GetWin(), false)));

  __connector->Add(rs);

  return static_cast<void*>(rs);
}

extern "C" EXPORT_API int
screen_connector_watcher_evas_remove(screen_connector_watcher_evas_h handle) {
  RemoteSurfaceWatcherStub* rs =
      static_cast<RemoteSurfaceWatcherStub*>(handle);

  __connector->Remove(rs);

  return 0;
}

extern "C" EXPORT_API int
screen_connector_watcher_evas_update(
    screen_connector_watcher_evas_h watcher_evas_h, const char* instance_id) {
  screen_connector::RemoteSurfaceWatcher* rs =
      static_cast<screen_connector::RemoteSurfaceWatcher*>(watcher_evas_h);

  if (!rs)
    return -1;

  return rs->SetRemoteRender();
}

extern "C" EXPORT_API int
screen_connector_watcher_evas_set_changed_event_filter(
    screen_connector_watcher_evas_h watcher_evas_h,
    screen_connector_evas_changed_event_filter_type type) {
  screen_connector::RemoteSurfaceWatcher* rs =
      static_cast<screen_connector::RemoteSurfaceWatcher*>(watcher_evas_h);

  if (!rs)
    return -1;

  rs->SetChangedEventFilter(
      static_cast<screen_connector::RemoteSurface::ChangedEventFilter>(type));

  return 0;
}
